@@ -3,6 +3,11 @@
 
 namespace ASCII {
 
+    template<class T>
+    inline bool isSign(T c) {
+        return c == '+' || c == '-';
+    }
+
     template<class T>
     static bool isLower(T c) {
         return c >= 'a' && c <= 'z';
@@ -28,6 +33,11 @@ namespace ASCII {
         return isAlpha(c) || isDigit(c);
     }
 
+    template<class T>
+    static bool isPrint(T c) {
+        return ' ' <= c && c <= '~';
+    }
+
     template<class T>
     static T toUpper(T c) {
         return isLower(c) ? (c - ' ') : c;
@@ -69,6 +79,15 @@ namespace ASCII {
             ++s;
         return s;
     }
+
+    template<class T>
+    static int hexDigit(T c) {
+        return c >= '0' && c <= '9' ? int(c - '0') :
+               c >= 'a' && c <= 'f' ? int(c - 'a') + 10 :
+               c >= 'A' && c <= 'F' ? int(c - 'A') + 10 :
+               -1;
+    }
+
 }
 
 #endif
