@@ -13,6 +13,11 @@
 #else
 #define OVERRIDE override
 #endif
+#if __cplusplus == 199711L
+#define nullptr NULL
+#define override
+#define final
+#endif
 
 /*** Essential Arithmetic Functions *******************************************/
 
@@ -103,7 +108,7 @@ void msg(char const *msg, ...) __attribute__((format(printf, 1, 2) ));
 void tlog(char const *msg, ...) __attribute__((format(printf, 1, 2) ));
 void precondition(const char *expr, const char *file, int line);
 char* path_lookup(const char* name);
-char* progpath(void);
+char* progpath();
 void show_backtrace(const int limit = 0);
 
 #define DEPRECATE(x) \
@@ -115,8 +120,6 @@ void show_backtrace(const int limit = 0);
 
 #define ACOUNT(x) (sizeof(x)/sizeof(x[0]))
 
-#define REDIR_ROOT(path) (path)
-
 //!!! clean these up
 #define KEY_MODMASK(x) ((x) & (xapp->KeyMask))
 #define BUTTON_MASK(x) ((x) & (xapp->ButtonMask))
@@ -252,6 +255,9 @@ char* load_text_file(const char *filename);
 
 #include "debug.h"
 
+extern bool loggingEvents;
+extern bool initLogEvents();
+
 void logAny(const union _XEvent& xev);
 void logButton(const union _XEvent& xev);
 void logClientMessage(const union _XEvent& xev);
@@ -271,10 +277,15 @@ void logUnmap(const union _XEvent& xev);
 void logMotion(const union _XEvent& xev);
 void logProperty(const union _XEvent& xev);
 void logReparent(const union _XEvent& xev);
+void logRandrScreen(const union _XEvent& xev);
+void logRandrNotify(const union _XEvent& xev);
 void logShape(const union _XEvent& xev);
 void logVisibility(const union _XEvent& xev);
 void logEvent(const union _XEvent& xev);
 
+typedef const char* (*AtomNameFunc)(unsigned long atom);
+void setAtomName(AtomNameFunc atomNameFunc);
+const char* getAtomName(unsigned long atom);
 void setLogEvent(int evtype, bool enable);
 bool toggleLogEvents();
 const char* eventName(int eventType);
