@@ -72,7 +72,7 @@
 #undef ENABLE_OSS
 #endif
 
-#ifdef HAVE_SNDFILE_H
+#if defined(ENABLE_ALSA) || defined(ENABLE_AO) || defined(ENABLE_OSS)
 #include <sndfile.h>
 #endif
 
@@ -124,7 +124,6 @@ public:
     virtual bool verbose() const = 0;
     virtual const char* alsaDevice() const = 0;
     virtual const char* ossDevice() const = 0;
-    virtual const char* esdServer() const = 0;
 
     /**
      * Finds a filename for sample with the specified gui event.
@@ -275,7 +274,7 @@ bool YALSAAudio::play(int sound) {
         const int N = 8*1024;
         short sbuf[N]; // period_size * channels * snd_pcm_format_width(format)) / 8
         for (int n; (n = sf_readf_short(sf, sbuf, N / 2)) > 0; ) {
-            if ((err = snd_pcm_writei(playback_handle, sbuf, n) != n)) {
+            if ((err = snd_pcm_writei(playback_handle, sbuf, n)) != n) {
                 warn("write to audio interface failed (%s) %zd %d\n",
                          snd_strerror(err), sizeof(short), n);
                 goto done;
@@ -534,9 +533,11 @@ done:
 
 class IceSound : public SoundConf {
 public:
-    IceSound(int argc, char** argv);
+    IceSound();
     virtual ~IceSound() {}
 
+    void parseArgs(int argc, char** argv);
+
     virtual bool verbose() const { return verbosity; }
     virtual const char* alsaDevice() const {
         return alsaDeviceFile ? alsaDeviceFile :
@@ -546,9 +547,6 @@ public:
         return ossDeviceFile ? ossDeviceFile :
             deviceFile ? deviceFile : OSS_DEFAULT_DEVICE;
     }
-    virtual const char* esdServer() const {
-        return esdServerName ? esdServerName : getenv("ESPEAKER");
-    }
     virtual char* findSample(int sound) const;
 
     int run();
@@ -560,7 +558,6 @@ private:
     char const* deviceFile;
     char const* alsaDeviceFile;
     char const* ossDeviceFile;
-    char const* esdServerName;
     char const* displayName;
     char const* interfaceNames;
     class YAudioInterface* audio;
@@ -589,13 +586,12 @@ private:
     static void hup(int sig);
 };
 
-IceSound::IceSound(int argc, char** argv) :
+IceSound::IceSound() :
     verbosity(false),
     sampleDir(0),
     deviceFile(0),
     alsaDeviceFile(0),
     ossDeviceFile(0),
-    esdServerName(0),
     displayName(0),
     interfaceNames(audio_interfaces),
     audio(0),
@@ -610,6 +606,10 @@ IceSound::IceSound(int argc, char** argv) :
 #endif
     initPaths();
     initSignals();
+}
+
+void IceSound::parseArgs(int argc, char** argv)
+{
     for (char **arg = argv + 1; arg < argv + argc; ++arg) {
         if (**arg == '-') {
             char* value(0);
@@ -632,7 +632,7 @@ IceSound::IceSound(int argc, char** argv) :
                 ossDeviceFile = value;
             }
             else if (GetArgument(value, "S", "server", arg, argv + argc)) {
-                esdServerName = value;
+                /*ignore*/;
             }
             else if (GetArgument(value, "z", "snooze", arg, argv + argc)) {
                 long t = strtol(value, NULL, 10);
@@ -980,7 +980,10 @@ int main(int argc, char *argv[]) {
     textdomain(PACKAGE);
 
     ApplicationName = my_basename(argv[0]);
-    return IceSound(argc, argv).run();
+
+    IceSound icesound;
+    icesound.parseArgs(argc, argv);
+    return icesound.run();
 }
 
 
