@@ -1,6 +1,12 @@
 #ifndef __REF_H
 #define __REF_H
 
+#if __cplusplus == 199711L
+#define nullptr NULL
+#define override
+#define final
+#endif
+
 class refcounted {
 public:
     int __refcount;
@@ -73,11 +79,11 @@ public:
     bool operator==(null_ref &) const { return ptr == 0; }
     bool operator!=(null_ref &) const { return ptr != 0; }
 
-    ref<T>& operator=(null_ref &) {
-        if (ptr)
+    void operator=(null_ref &) {
+        if (ptr) {
             __unref();
-        ptr = 0;
-        return *this;
+            ptr = 0;
+        }
     }
     T *_ptr() const { return ptr; }
 };
@@ -110,6 +116,13 @@ private:
     operator void*();
 };
 
+template<class T>
+class lazily : public lazy<T> {
+public:
+    operator bool() { return true; }
+    void operator=(null_ref&) { lazy<T>::operator=(null); }
+};
+
 template<class T>
 bool operator==(T* q, lazy<T>& p) { return q == p._ptr(); }
 template<class T>
