@@ -1,17 +1,11 @@
 #ifndef __WMFRAME_H
 #define __WMFRAME_H
 
-#include "ywindow.h"
-#include "ymenu.h"
-#include "ytimer.h"
 #include "ymsgbox.h"
-#include "yaction.h"
-#include "wmclient.h"
-#include "wmbutton.h"
 #include "wmoption.h"
-#include "WinMgr.h"
 #include "wmmgr.h"
 #include "yicon.h"
+#include "ylist.h"
 
 class YClientContainer;
 class MiniIcon;
@@ -19,9 +13,22 @@ class TaskBarApp;
 class TrayApp;
 class YFrameTitleBar;
 
-class YFrameWindow: public YWindow, public YActionListener, public YTimerListener, public YPopDownListener, public YMsgBoxListener, public ClientData {
+class YFrameWindow:
+    public YWindow,
+    public YActionListener,
+    public YTimerListener,
+    public YPopDownListener,
+    public YMsgBoxListener,
+    public ClientData,
+    public YLayeredNode,
+    public YCreatedNode,
+    public YFocusedNode
+{
 public:
-    YFrameWindow(YActionListener *wmActionListener, YWindow *parent);
+    YFrameWindow(YActionListener *wmActionListener,
+                 unsigned depth = CopyFromParent,
+                 Visual* visual = nullptr,
+                 Colormap clmap = CopyFromParent);
     virtual ~YFrameWindow();
 
     void doManage(YFrameClient *client, bool &doActivate, bool &requestFocus);
@@ -30,12 +37,13 @@ public:
     void unmanage(bool reparent = true);
     void sendConfigure();
 
+    Window createPointerWindow(Cursor cursor, Window parent);
     void createPointerWindows();
     void grabKeys();
 
     void focus(bool canWarp = false);
-    void activate(bool canWarp = false);
-    void activateWindow(bool raise);
+    void activate(bool canWarp = false, bool curWork = true);
+    void activateWindow(bool raise, bool curWork = true);
 
     virtual void paint(Graphics &g, const YRect &r);
 
@@ -52,9 +60,10 @@ public:
 
     virtual bool handleTimer(YTimer *t);
 
-    virtual void actionPerformed(YAction *action, unsigned int modifiers);
+    virtual void actionPerformed(YAction action, unsigned int modifiers);
     virtual void handleMsgBox(YMsgBox *msgbox, int operation);
-    
+    virtual YFrameWindow* frame() { return this; }
+
     void wmRestore();
     void wmMinimize();
     void wmMaximize();
@@ -69,22 +78,20 @@ public:
     void doRaise();
     void wmClose();
     void wmConfirmKill();
+    static YMsgBox* wmConfirmKill(const ustring& title, YMsgBoxListener *recvr);
     void wmKill();
     void wmNextWindow();
     void wmPrevWindow();
-    void wmLastWindow();
     void wmMove();
     void wmSize();
     void wmOccupyAll();
     void wmOccupyAllOrCurrent();
-    void wmOccupyWorkspace(long workspace);
-    void wmOccupyOnlyWorkspace(long workspace);
-    void wmMoveToWorkspace(long workspace);
+    void wmOccupyWorkspace(int workspace);
+    void wmOccupyOnlyWorkspace(int workspace);
+    void wmMoveToWorkspace(int workspace);
     void wmSetLayer(long layer);
-#ifdef CONFIG_TRAY
     void wmSetTrayOption(long option);
     void wmToggleTray();
-#endif
 #if DO_NOT_COVER_OLD
     void wmToggleDoNotCover();
 #endif
@@ -103,15 +110,12 @@ public:
     void updateFocusOnMap(bool &doActivate);
 
     YFrameClient *client() const { return fClient; }
-    YFrameTitleBar *titlebar() const { return fTitleBar; }
+    YFrameTitleBar *titlebar();
     YClientContainer *container() const { return fClientContainer; }
 
-#ifdef WMSPEC_HINTS
-    void startMoveSize(int x, int y,
-                                     int direction);
-#endif
+    void startMoveSize(int x, int y, int direction);
 
-    void startMoveSize(int doMove, int byMouse,
+    void startMoveSize(bool doMove, bool byMouse,
                        int sideX, int sideY,
                        int mouseXroot, int mouseYroot);
     void endMoveSize();
@@ -133,34 +137,32 @@ public:
 
     void outlineMove();
     void outlineResize();
-    
+
     void constrainPositionByModifier(int &x, int &y, const XMotionEvent &motion);
     void constrainMouseToWorkspace(int &x, int &y);
 
     void getDefaultOptions(bool &doActivate);
 
     bool canSize(bool boriz = true, bool vert = true);
-    bool canMove();
-    bool canClose();
-    bool canMaximize();
-    bool canMinimize();
-    bool canRollup();
-    bool canHide();
-    bool canLower();
+    bool canMove() const;
+    bool canClose() const;
+    bool canMaximize() const;
+    bool canMinimize() const;
+    bool canRestore() const;
+    bool canRollup() const;
+    bool canHide() const;
+    bool canLower() const;
     bool canRaise();
     bool canFullscreen() { return true; }
-    bool Overlaps(bool below);
+    bool overlaps(bool below);
+    unsigned overlap(YFrameWindow *other);
 
     void insertFrame(bool top);
     void removeFrame();
     void setAbove(YFrameWindow *aboveFrame); // 0 = at the bottom
     void setBelow(YFrameWindow *belowFrame); // 0 = at the top
-    YFrameWindow *next() const { return fNextFrame; }
-    YFrameWindow *prev() const { return fPrevFrame; }
-    void setNext(YFrameWindow *next) { fNextFrame = next; }
-    void setPrev(YFrameWindow *prev) { fPrevFrame = prev; }
 
-    typedef enum {
+    enum FindWindowFlags {
         fwfVisible    = 1 << 0, // visible windows only
         fwfCycle      = 1 << 1, // cycle when bottom(top) reached
         fwfBackward   = 1 << 2, // go up in zorder (default=down)
@@ -174,18 +176,12 @@ public:
         fwfUnminimized = 1 << 10, // normal/rolledup only
         fwfHidden     = 1 << 11, // hidden
         fwfNotHidden  = 1 << 12 // not hidden
-    } FindWindowFlags;
+    };
 
     YFrameWindow *findWindow(int flag);
 
-    YFrameButton *menuButton() const { return fMenuButton; }
-    YFrameButton *closeButton() const { return fCloseButton; }
-    YFrameButton *minimizeButton() const { return fMinimizeButton; }
-    YFrameButton *maximizeButton() const { return fMaximizeButton; }
-    YFrameButton *hideButton() const { return fHideButton; }
-    YFrameButton *rollupButton() const { return fRollupButton; }
-    YFrameButton *depthButton() const { return fDepthButton; }
     void updateMenu();
+    virtual void updateSubmenus();
 
     virtual void raise();
     virtual void lower();
@@ -196,8 +192,8 @@ public:
     virtual void popupSystemMenu(YWindow *owner);
     virtual void handlePopDown(YPopupWindow *popup);
 
-    virtual void configure(const YRect &r);
-    
+    virtual void configure(const YRect2& r);
+
     void getNewPos(const XConfigureRequestEvent &cr,
                    int &cx, int &cy, int &cw, int &ch);
     void configureClient(const XConfigureRequestEvent &configureRequest);
@@ -207,7 +203,7 @@ public:
     void setShape();
 #endif
 
-    enum {
+    enum YFrameFunctions {
         ffMove          = (1 << 0),
         ffResize        = (1 << 1),
         ffClose         = (1 << 2),
@@ -215,9 +211,9 @@ public:
         ffMaximize      = (1 << 4),
         ffHide          = (1 << 5),
         ffRollup        = (1 << 6)
-    } YFrameFunctions;
+    };
 
-    enum {
+    enum YFrameDecors {
         fdTitleBar      = (1 << 0),
         fdSysMenu       = (1 << 1),
         fdBorder        = (1 << 2),
@@ -228,48 +224,49 @@ public:
         fdHide          = (1 << 7),
         fdRollup        = (1 << 8),
         fdDepth         = (1 << 9)
-    } YFrameDecors;
-
-    /// !!! needs refactoring (some are not optional right now)
-    /// should be #ifndef NO_WINDOW_OPTIONS
-
-    enum YFrameOptions {
-        foAllWorkspaces         = (1 << 0),
-        foIgnoreTaskBar         = (1 << 1),
-        foIgnoreWinList         = (1 << 2),
-        foFullKeys              = (1 << 3),
-        foIgnoreQSwitch         = (1 << 4),
-        foNoFocusOnAppRaise     = (1 << 5),
-        foIgnoreNoFocusHint     = (1 << 6),
-        foIgnorePosition        = (1 << 7),
-        foDoNotCover            = (1 << 8),
-        foFullscreen            = (1 << 9),
-        foMaximizedVert         = (1 << 10),
-        foMaximizedHorz         = (1 << 11),
-        foNonICCCMConfigureRequest = (1 << 12),
-        foMinimized             = (1 << 13),
-        foDoNotFocus            = (1 << 14),
-        foForcedClose           = (1 << 15),
-        foNoFocusOnMap          = (1 << 16),
-        foNoIgnoreTaskBar       = (1 << 17),
-        foAppTakesFocus         = (1 << 18)
     };
 
-    unsigned long frameFunctions() const { return fFrameFunctions; }
-    unsigned long frameDecors() const { return fFrameDecors; }
-    unsigned long frameOptions() const { return fFrameOptions; }
+    enum YFrameOptions {
+        foAllWorkspaces            = (1 << 0),  // WinStateSticky
+        foMinimized                = (1 << 1),  // WinStateMinimized
+        foMaximizedVert            = (1 << 2),  // WinStateMaximizedVert
+        foMaximizedHorz            = (1 << 3),  // WinStateMaximizedHoriz
+        foMaximizedBoth            = (3 << 2),  // WinStateMaximizedBoth
+        foAppTakesFocus            = (1 << 4),
+        foDoNotCover               = (1 << 5),
+        foDoNotFocus               = (1 << 6),
+        foForcedClose              = (1 << 7),
+        foFullKeys                 = (1 << 8),
+        foFullscreen               = (1 << 9),
+        foIgnoreNoFocusHint        = (1 << 10),
+        foIgnorePagerPreview       = (1 << 11),
+        foIgnorePosition           = (1 << 12),
+        foIgnoreQSwitch            = (1 << 13),
+        foIgnoreTaskBar            = (1 << 14),
+        foIgnoreUrgent             = (1 << 15),
+        foIgnoreWinList            = (1 << 16),
+        foNoFocusOnAppRaise        = (1 << 17),
+        foNoFocusOnMap             = (1 << 18),
+        foNoIgnoreTaskBar          = (1 << 19),
+        foNonICCCMConfigureRequest = (1 << 20),
+        foClose                    = (1 << 21),
+    };
+
+    unsigned frameFunctions() const { return fFrameFunctions; }
+    unsigned frameDecors() const { return fFrameDecors; }
+    unsigned frameOptions() const { return fFrameOptions; }
+    void updateAllowed();
+    void updateNetWMState();
     void getFrameHints();
-#ifndef NO_WINDOW_OPTIONS
     void getWindowOptions(WindowOption &opt, bool remove); /// !!! fix kludges
     void getWindowOptions(WindowOptions *list, WindowOption &opt, bool remove);
-#endif
 
     YMenu *windowMenu();
 
     long getState() const { return fWinState; }
     void setState(long mask, long state);
 
-    bool isFullscreen() const { return (getState() & WinStateFullscreen) ? true : false; }
+    bool isFullscreen() const { return (getState() & WinStateFullscreen); }
 
     int borderXN() const;
     int borderYN() const;
@@ -278,22 +275,16 @@ public:
     int borderX() const;
     int borderY() const;
     int titleY() const;
-    
+
     void layoutTitleBar();
-    void layoutButtons();
     void layoutResizeIndicators();
     void layoutShape();
     void layoutClient();
 
-    //void workspaceShow();
-    //void workspaceHide();
-
     YFrameWindow *nextLayer();
     YFrameWindow *prevLayer();
-#ifdef CONFIG_WINLIST
     WindowListItem *winListItem() const { return fWinListItem; }
     void setWinListItem(WindowListItem *i) { fWinListItem = i; }
-#endif
 
     void addAsTransient();
     void removeAsTransient();
@@ -308,10 +299,8 @@ public:
     YFrameWindow *owner() const { return fOwner; }
     YFrameWindow *mainOwner();
 
-#ifndef LITE
     ref<YIcon> getClientIcon() const { return fFrameIcon; }
     ref<YIcon> clientIcon() const;
-#endif
 
     void getNormalGeometryInner(int *x, int *y, int *w, int *h);
     void setNormalGeometryOuter(int x, int y, int w, int h);
@@ -325,35 +314,47 @@ public:
 
     void updateTitle();
     void updateIconTitle();
-#ifndef LITE
     void updateIcon();
-#endif
     void updateState();
     void updateLayer(bool restack = true);
     //void updateWorkspace();
     void updateLayout();
+    void updateExtents();
     void performLayout();
 
     void updateMwmHints();
     void updateProperties();
-#ifdef CONFIG_TASKBAR
     void updateTaskBar();
-#endif
 
-    void setTypeDesktop(bool typeDesktop) { fTypeDesktop = typeDesktop; }
-    void setTypeDock(bool typeDock) { fTypeDock = typeDock; }
-    void setTypeSplash(bool typeSplash) { fTypeSplash = typeSplash; }
-    bool isTypeDock() { return fTypeDock; }
+    enum WindowType {
+        wtCombo,
+        wtDesktop,
+        wtDialog,
+        wtDND,
+        wtDock,
+        wtDropdownMenu,
+        wtMenu,
+        wtNormal,
+        wtNotification,
+        wtPopupMenu,
+        wtSplash,
+        wtToolbar,
+        wtTooltip,
+        wtUtility
+    };
 
-    long getWorkspace() const { return fWinWorkspace; }
-    void setWorkspace(long workspace);
+    void setWindowType(enum WindowType winType) { fWindowType = winType; }
+    bool isTypeDock() { return (fWindowType == wtDock); }
+
+    int getWorkspace() const { return fWinWorkspace; }
+    int getTrayOrder() const { return fTrayOrder; }
+    void setWorkspace(int workspace);
     void setWorkspaceHint(long workspace);
     long getActiveLayer() const { return fWinActiveLayer; }
     void setRequestedLayer(long layer);
-#ifdef CONFIG_TRAY
+    long getRequestedLayer() const { return fWinRequestedLayer; }
     long getTrayOption() const { return fWinTrayOption; }
     void setTrayOption(long option);
-#endif
     void setDoNotCover(bool flag);
     bool isMaximized() const { return (getState() &
                                  (WinStateMaximizedHoriz |
@@ -363,9 +364,11 @@ public:
     bool isMaximizedFully() const { return isMaximizedVert() && isMaximizedHoriz(); }
     bool isMinimized() const { return (getState() & WinStateMinimized) ? true : false; }
     bool isHidden() const { return (getState() & WinStateHidden) ? true : false; }
+    bool isSkipPager() const { return (getState() & WinStateSkipPager) ? true : false; }
     bool isSkipTaskBar() const { return (getState() & WinStateSkipTaskBar) ? true : false; }
     bool isRollup() const { return (getState() & WinStateRollup) ? true : false; }
-    bool isSticky() const { return (getState() & WinStateAllWorkspaces) ? true : false; }
+    bool isSticky() const { return (getState() & WinStateSticky) ? true : false; }
+    bool isAllWorkspaces() const { return (getWorkspace() == -1) ? true : false; }
     //bool isHidWorkspace() { return (getState() & WinStateHidWorkspace) ? true : false; }
     //bool isHidTransient() { return (getState() & WinStateHidTransient) ? true : false; }
 
@@ -374,15 +377,15 @@ public:
 
     bool isIconic() const { return isMinimized() && fMiniIcon; }
 
-    MiniIcon *getMiniIcon() const { return fMiniIcon; }
+    MiniIcon *getMiniIcon();
 
     bool isManaged() const { return fManaged; }
     void setManaged(bool isManaged) { fManaged = isManaged; }
 
-    void setSticky(bool sticky);
+    void setAllWorkspaces();
 
-    bool visibleOn(long workspace) const {
-        return (isSticky() || getWorkspace() == workspace);
+    bool visibleOn(int workspace) const {
+        return (isAllWorkspaces() || getWorkspace() == workspace);
     }
     bool visibleNow() const { return visibleOn(manager->activeWorkspace()); }
 
@@ -400,38 +403,24 @@ public:
         return (frameOptions() & foDoNotCover) ? true : false;
     }
 
-#ifndef LITE
     virtual ref<YIcon> getIcon() const { return clientIcon(); }
-#endif
 
     virtual ustring getTitle() const { return client()->windowTitle(); }
     virtual ustring getIconTitle() const { return client()->iconTitle(); }
 
-    YFrameButton *getButton(char c);
-    void positionButton(YFrameButton *b, int &xPos, bool onRight);
-    bool isButton(char c);
-
-#ifdef WMSPEC_HINTS
     void updateNetWMStrut();
-#endif
+    void updateNetWMStrutPartial();
+    void updateNetStartupId();
+    void updateNetWMUserTime();
+    void updateNetWMUserTimeWindow();
+    void updateNetWMWindowOpacity();
+    void updateNetWMFullscreenMonitors(int, int, int, int);
+
     int strutLeft() { return fStrutLeft; }
     int strutRight() { return fStrutRight; }
     int strutTop() { return fStrutTop; }
     int strutBottom() { return fStrutBottom; }
-
-    YFrameWindow *nextCreated() { return fNextCreatedFrame; }
-    YFrameWindow *prevCreated() { return fPrevCreatedFrame; }
-    void setNextCreated(YFrameWindow *f) { fNextCreatedFrame = f; }
-    void setPrevCreated(YFrameWindow *f) { fPrevCreatedFrame = f; }
-
-    YFrameWindow *nextFocus() { return fNextFocusFrame; }
-    YFrameWindow *prevFocus() { return fPrevFocusFrame; }
-    void setNextFocus(YFrameWindow *f) { fNextFocusFrame = f; }
-    void setPrevFocus(YFrameWindow *f) { fPrevFocusFrame = f; }
-
-    void insertFocusFrame(bool focus);
-    void insertLastFocusFrame();
-    void removeFocusFrame();
+    bool haveStruts() const { return fHaveStruts; }
 
     void updateUrgency();
     void setWmUrgency(bool wmUrgency);
@@ -442,6 +431,16 @@ public:
     long getOldLayer() { return fOldLayer; }
     void saveOldLayer() { fOldLayer = fWinActiveLayer; }
 
+    bool hasIndicators() const { return indicatorsCreated; }
+    Window topSideIndicator() const { return topSide; }
+    Window topLeftIndicator() const { return topLeft; }
+    Window topRightIndicator() const { return topRight; }
+    Time since() const { return fStartManaged; }
+    bool startMinimized() const { return hasbit(fFrameOptions, foMinimized); }
+
+    void addToWindowList();
+    void removeFromWindowList();
+
 private:
     /*typedef enum {
         fsMinimized       = 1 << 0,
@@ -451,81 +450,61 @@ private:
         fsWorkspaceHidden = 1 << 4
     } FrameStateFlags;*/
 
+    bool fManaged;
     bool fFocused;
-    unsigned long fFrameFunctions;
-    unsigned long fFrameDecors;
-    unsigned long fFrameOptions;
+    unsigned fFrameFunctions;
+    unsigned fFrameDecors;
+    unsigned fFrameOptions;
 
     int normalX, normalY, normalW, normalH;
     int posX, posY, posW, posH;
+    int extentLeft, extentRight, extentTop, extentBottom;
 
     int iconX, iconY;
 
     YFrameClient *fClient;
     YClientContainer *fClientContainer;
     YFrameTitleBar *fTitleBar;
-    YFrameButton *fCloseButton;
-    YFrameButton *fMenuButton;
-    YFrameButton *fMaximizeButton;
-    YFrameButton *fMinimizeButton;
-    YFrameButton *fHideButton;
-    YFrameButton *fRollupButton;
-    YFrameButton *fDepthButton;
 
     YPopupWindow *fPopupActive;
 
     int buttonDownX, buttonDownY;
     int grabX, grabY;
-    int movingWindow, sizingWindow;
-    int sizeByMouse;
+    bool movingWindow, sizingWindow;
     int origX, origY, origW, origH;
 
-    YFrameWindow *fNextFrame; // window below this one
-    YFrameWindow *fPrevFrame; // window above this one
-
-    YFrameWindow *fNextCreatedFrame;
-    YFrameWindow *fPrevCreatedFrame;
-
-    YFrameWindow *fNextFocusFrame;
-    YFrameWindow *fPrevFocusFrame;
-
     Window topSide, leftSide, rightSide, bottomSide;
-    Window topLeftCorner, topRightCorner, bottomLeftCorner, bottomRightCorner;
-    int indicatorsVisible;
+    Window topLeft, topRight, bottomLeft, bottomRight;
+    bool indicatorsCreated;
+    bool indicatorsVisible;
 
-#ifdef CONFIG_TASKBAR
     TaskBarApp *fTaskBarApp;
-#endif
-#ifdef CONFIG_TRAY
     TrayApp *fTrayApp;
-#endif
     MiniIcon *fMiniIcon;
-#ifdef CONFIG_WINLIST
     WindowListItem *fWinListItem;
-#endif
-#ifndef LITE
     ref<YIcon> fFrameIcon;
-#endif
 
     YFrameWindow *fOwner;
     YFrameWindow *fTransient;
     YFrameWindow *fNextTransient;
     YActionListener *wmActionListener;
 
-    static YTimer *fAutoRaiseTimer;
-    static YTimer *fDelayFocusTimer;
+    static lazy<YTimer> fAutoRaiseTimer;
+    static lazy<YTimer> fDelayFocusTimer;
 
-    long fWinWorkspace;
+    int fWinWorkspace;
     long fWinRequestedLayer;
     long fWinActiveLayer;
-#ifdef CONFIG_TRAY
     long fWinTrayOption;
-#endif
     long fWinState;
-    long fWinStateMask;
-    bool fManaged;
     long fWinOptionMask;
     long fOldLayer;
+    int fTrayOrder;
+
+    int fFullscreenMonitorsTop;
+    int fFullscreenMonitorsBottom;
+    int fFullscreenMonitorsLeft;
+    int fFullscreenMonitorsRight;
 
     YMsgBox *fKillMsgBox;
 
@@ -535,17 +514,22 @@ private:
     int fStrutTop;
     int fStrutBottom;
 
-    int fShapeWidth;
-    int fShapeHeight;
+    // _NET_WM_USER_TIME support
+    UserTime fUserTime;
+    Window fUserTimeWindow;
+    Time fStartManaged;
+
+    unsigned fShapeWidth;
+    unsigned fShapeHeight;
     int fShapeTitleY;
     int fShapeBorderX;
     int fShapeBorderY;
 
+    bool fHaveStruts;
     bool fWmUrgency;
     bool fClientUrgency;
-    bool fTypeDesktop;
-    bool fTypeDock;
-    bool fTypeSplash;
+
+    enum WindowType fWindowType;
 
     enum WindowArranges {
         waTop,
@@ -570,42 +554,11 @@ private:
     void setSize(int, int);
     void setWindowGeometry(const YRect &r) {
         YWindow::setGeometry(r);
-        performLayout();
     }
     friend class MiniIcon;
 };
 
-//!!! remove this
-extern ref<YPixmap> frameTL[2][2];
-extern ref<YPixmap> frameT[2][2];
-extern ref<YPixmap> frameTR[2][2];
-extern ref<YPixmap> frameL[2][2];
-extern ref<YPixmap> frameR[2][2];
-extern ref<YPixmap> frameBL[2][2];
-extern ref<YPixmap> frameB[2][2];
-extern ref<YPixmap> frameBR[2][2];
-
-extern ref<YPixmap> titleJ[2];
-extern ref<YPixmap> titleL[2];
-extern ref<YPixmap> titleS[2];
-extern ref<YPixmap> titleP[2];
-extern ref<YPixmap> titleT[2];
-extern ref<YPixmap> titleM[2];
-extern ref<YPixmap> titleB[2];
-extern ref<YPixmap> titleR[2];
-extern ref<YPixmap> titleQ[2];
-
-extern ref<YPixmap> menuButton[3];
-
-#ifdef CONFIG_GRADIENTS
-extern ref<YImage> rgbFrameT[2][2];
-extern ref<YImage> rgbFrameL[2][2];
-extern ref<YImage> rgbFrameR[2][2];
-extern ref<YImage> rgbFrameB[2][2];
-extern ref<YImage> rgbTitleS[2];
-extern ref<YImage> rgbTitleT[2];
-extern ref<YImage> rgbTitleB[2];
 #endif
 
-#endif
 
+// vim: set sw=4 ts=4 et:
