@@ -1,8 +1,6 @@
 #ifndef __WINLIST_H
 #define __WINLIST_H
 
-#ifdef CONFIG_WINLIST
-
 #include "wmclient.h" // !!! should be ywindow
 #include "ylistbox.h"
 #include "yscrollview.h"
@@ -20,7 +18,7 @@ public:
     virtual ~WindowListItem();
 
     virtual int getOffset();
-    
+
     virtual ustring getText();
     virtual ref<YIcon> getIcon();
     ClientData *getFrame() const { return fFrame; }
@@ -37,9 +35,9 @@ public:
 
     virtual bool handleKey(const XKeyEvent &key);
     virtual void handleClick(const XButtonEvent &up, int count);
-    
+
     virtual void activateItem(YListItem *item);
-    virtual void actionPerformed(YAction *action, unsigned int modifiers);
+    virtual void actionPerformed(YAction action, unsigned int modifiers);
 
     void enableCommands(YMenu *popup);
     void getSelectedWindows(YArray<YFrameWindow *> &frames);
@@ -50,32 +48,57 @@ public:
     WindowList(YWindow *aParent, YActionListener *wmActionListener);
     virtual ~WindowList();
 
-    void handleFocus(const XFocusChangeEvent &focus);
+    void updateWorkspaces();
+
+    virtual void handleFocus(const XFocusChangeEvent &focus);
+    virtual void handleExpose(const XExposeEvent&) {}
     virtual void handleClose();
 
-    virtual void configure(const YRect &r);
+    virtual void configure(const YRect2 &r);
     void relayout();
 
     WindowListItem *addWindowListApp(YFrameWindow *frame);
     void removeWindowListApp(WindowListItem *item);
     void updateWindowListApp(WindowListItem *item);
+    void updateWindowListApps();
 
     void repaintItem(WindowListItem *item) { list->repaintItem(item); }
     void showFocused(int x, int y);
 
     WindowListBox *getList() const { return list; }
+    YMenu* getWindowListPopup();
+    YMenu* getWindowListAllPopup();
 
 private:
-    WindowListBox *list;
-    YScrollView *scroll;
+    long fWorkspaceCount;
     WindowListItem **workspaceItem;
     YActionListener *wmActionListener;
+    YMenu* windowListPopup;
+    YMenu* windowListAllPopup;
+    YScrollView *scroll;
+    WindowListBox *list;
 
+    void setupClient();
     void insertApp(WindowListItem *item);
 };
 
-extern WindowList *windowList;
+extern class WindowListProxy {
+    WindowList* wlist;
+public:
+    WindowListProxy() : wlist(nullptr) { }
+    ~WindowListProxy() { release(); }
+    operator bool() { return wlist; }
+    operator WindowList*() { return wlist ? wlist : acquire(); }
+    WindowList* operator->() { return *this; }
+    WindowList* _ptr() { return wlist; }
+    void release() { if (wlist) { delete wlist; wlist = nullptr; } }
+    void operator=(null_ref&) { release(); }
+private:
+    WindowList* acquire();
+    operator void* ();  // undefined
+    operator int ();    // undefined
+} windowList;
 
 #endif
 
-#endif
+// vim: set sw=4 ts=4 et:
