@@ -7,9 +7,6 @@
  */
 #include "config.h"
 
-#ifndef LITE
-
-#include "ypixbuf.h"
 #include "ykey.h"
 #include "ylistbox.h"
 #include "yrect.h"
@@ -24,10 +21,10 @@
 #include <string.h>
 
 static ref<YFont> listBoxFont;
-static YColor *listBoxBg = 0;
-static YColor *listBoxFg = 0;
-static YColor *listBoxSelBg = 0;
-static YColor *listBoxSelFg = 0;
+static YColorName listBoxBg(&clrListBox);
+static YColorName listBoxFg(&clrListBoxText);
+static YColorName listBoxSelBg(&clrListBoxSelected);
+static YColorName listBoxSelFg(&clrListBoxSelectedText);
 
 int YListBox::fAutoScrollDelta = 0;
 
@@ -71,22 +68,19 @@ int YListItem::getOffset() {
     return 0;
 }
 
-YListBox::YListBox(YScrollView *view, YWindow *aParent): 
-    YWindow(aParent) INIT_GRADIENT(fGradient, NULL) {
+YListBox::YListBox(YScrollView *view, YWindow *aParent):
+    YWindow(aParent),
+    fGraphics(this),
+    fGradient(null)
+{
     if (listBoxFont == null)
         listBoxFont = YFont::getFont(XFA(listBoxFontName));
-    if (listBoxBg == 0)
-        listBoxBg = new YColor(clrListBox);
-    if (listBoxFg == 0)
-        listBoxFg = new YColor(clrListBoxText);
-    if (listBoxSelBg == 0)
-        listBoxSelBg = new YColor(clrListBoxSelected);
-    if (listBoxSelFg == 0)
-        listBoxSelFg = new YColor(clrListBoxSelectedText);
+    addStyle(wsNoExpose);
     setBitGravity(NorthWestGravity);
+    addEventMask(VisibilityChangeMask);
     fView = view;
     if (fView) {
-        fVerticalScroll = view->getVerticalScrollBar();;
+        fVerticalScroll = view->getVerticalScrollBar();
         fHorizontalScroll = view->getHorizontalScrollBar();
     } else {
         fHorizontalScroll = 0;
@@ -103,23 +97,24 @@ YListBox::YListBox(YScrollView *view, YWindow *aParent):
     fSelectStart = fSelectEnd = -1;
     fDragging = false;
     fSelect = false;
+    fVisible = false;
+    fOutdated = false;
     fItemCount = 0;
     fItems = 0;
+    setTitle("ListBox");
 }
 
 YListBox::~YListBox() {
     fFirst = fLast = 0;
     freeItems();
-#ifdef CONFIG_GRADIENTS
     fGradient = null;
-#endif
 }
 
 bool YListBox::isFocusTraversable() {
     return true;
 }
 
-int YListBox::addAfter(YListItem *prev, YListItem *item) {
+void YListBox::addAfter(YListItem *prev, YListItem *item) {
     PRECONDITION(item->getPrev() == 0);
     PRECONDITION(item->getNext() == 0);
 
@@ -134,11 +129,10 @@ int YListBox::addAfter(YListItem *prev, YListItem *item) {
     item->setPrev(prev);
     prev->setNext(item);
     fItemCount++;
-    repaint();
-    return 1;
+    outdated();
 }
 
-int YListBox::addItem(YListItem *item) {
+void YListBox::addItem(YListItem *item) {
     PRECONDITION(item->getPrev() == 0);
     PRECONDITION(item->getNext() == 0);
 
@@ -152,13 +146,14 @@ int YListBox::addItem(YListItem *item) {
         fFirst = item;
     fLast = item;
     fItemCount++;
-    repaint();
-    return 1;
+    outdated();
 }
 
 void YListBox::removeItem(YListItem *item) {
     freeItems();
 
+    bool focused = (inrange(fFocusedItem, 0, getItemCount() - 1)
+                    && fItems && fItems[fFocusedItem] == item);
     if (item->getPrev())
         item->getPrev()->setNext(item->getNext());
     else
@@ -171,7 +166,10 @@ void YListBox::removeItem(YListItem *item) {
     item->setPrev(0);
     item->setNext(0);
     fItemCount--;
-    repaint();
+
+    if (focused || fFocusedItem >= fItemCount)
+        fFocusedItem = -1;
+    outdated();
 }
 
 void YListBox::freeItems() {
@@ -264,7 +262,7 @@ void YListBox::ensureVisibility(int item) { //!!! horiz too
             oy = fy;
         if (oy != fOffsetY) {
             fOffsetY = oy;
-            repaint();///!!!fix (use scroll)
+            outdated();///!!!fix (use scroll)
         }
     }
 }
@@ -283,21 +281,51 @@ void YListBox::focusVisible() {
     }
 }
 
-void YListBox::configure(const YRect &r) {
-    YWindow::configure(r);
-
+void YListBox::configure(const YRect2& r) {
     resetScrollBars();
 
-#ifdef CONFIG_GRADIENTS
     if (listbackPixbuf != null
         && !(fGradient != null &&
              fGradient->width() == r.width() &&
              fGradient->height() == r.height()))
     {
         fGradient = listbackPixbuf->scale(r.width(), r.height());
-        repaint();
     }
-#endif
+    if (r.resized()) {
+        if (r.enlarged()) {
+            repaint();
+        } else {
+            outdated();
+        }
+    }
+}
+
+void YListBox::handleVisibility(const XVisibilityEvent& visib) {
+    bool prev = fVisible;
+    fVisible = (visib.state != VisibilityFullyObscured);
+    if (prev < fVisible)
+        repaint();
+}
+
+bool YListBox::handleTimer(YTimer* timer) {
+    if (timer == fTimer) {
+        if (fOutdated && fVisible) {
+            repaint();
+        }
+    }
+    return false;
+}
+
+void YListBox::outdated() {
+    fOutdated = true;
+    fTimer->setTimer(20L, this, true);
+}
+
+void YListBox::repaint() {
+    if (fVisible) {
+        fGraphics.paint();
+        fOutdated = false;
+    }
 }
 
 bool YListBox::handleKey(const XKeyEvent &key) {
@@ -359,29 +387,29 @@ bool YListBox::handleKey(const XKeyEvent &key) {
                                clear, extend, false);
             break;
         }
-#if 0
-        case XK_Prior:
-            fVerticalScroll->setValue(fVerticalScroll->getValue() -
-                                      fVerticalScroll->getBlockIncrement());
-            fOffsetY = fVerticalScroll->getValue();
-            fFocusedItem -= height() / getLineHeight();
-            if (fFocusedItem < 0)
-                if (count > 0)
-                    fFocusedItem = 0;
-                else
-                    fFocusedItem = -1;
-            repaint();
+        case XK_Prior: {
+            int const oldFocus(fFocusedItem);
+
+            focusVisible();
+            if (fFocusedItem > 0)
+                setFocusedItem(oldFocus == fFocusedItem
+                               ? max(0, fFocusedItem - 10)
+                               : fFocusedItem,
+                               clear, extend, false);
+
             break;
-        case XK_Next:
-            fVerticalScroll->setValue(fVerticalScroll->getValue() +
-                                      fVerticalScroll->getBlockIncrement());
-            fOffsetY = fVerticalScroll->getValue();
-            fFocusedItem += height() / getLineHeight();
-            if (fFocusedItem > count - 1)
-                fFocusedItem = count - 1;
-            repaint();
+        }
+        case XK_Next: {
+            int const oldFocus(fFocusedItem);
+
+            focusVisible();
+            if (fFocusedItem < getItemCount() - 1)
+                setFocusedItem(oldFocus == fFocusedItem
+                               ? min(getItemCount() - 1, fFocusedItem + 10)
+                               : fFocusedItem,
+                               clear, extend, false);
             break;
-#endif
+        }
         case 'a':
         case '/':
         case '\\':
@@ -567,22 +595,20 @@ void YListBox::paintItem(Graphics &g, int n) {
             s = fSelect;
     }
 
+    g.clearArea(0, y - fOffsetY, width(), lh);
     if (s) {
         g.setColor(listBoxSelBg);
         g.fillRect(0, y - fOffsetY, width(), lh);
     } else {
-#ifdef CONFIG_GRADIENTS
         if (fGradient != null)
             g.drawImage(fGradient, 0, y - fOffsetY, width(), lh,
                          0, y - fOffsetY);
-        else 
-#endif
-            if (listbackPixmap != null)
-                g.fillPixmap(listbackPixmap, 0, y - fOffsetY, width(), lh);
-            else {
-                g.setColor(listBoxBg);
-                g.fillRect(0, y - fOffsetY, width(), lh);
-            }
+        else if (listbackPixmap != null)
+            g.fillPixmap(listbackPixmap, 0, y - fOffsetY, width(), lh);
+        else {
+            g.setColor(listBoxBg);
+            g.fillRect(0, y - fOffsetY, width(), lh);
+        }
     }
 
     if (fFocusedItem == n) {
@@ -599,9 +625,14 @@ void YListBox::paintItem(Graphics &g, int n) {
     }
 
     ref<YIcon> icon = a->getIcon();
-
-    if (icon != null)
-        icon->draw(g, xpos + x - fOffsetX, y - fOffsetY + 1, YIcon::smallSize());
+    if (icon != null) {
+        ref<YImage> scaled = icon->getScaledIcon(YIcon::menuSize());
+        if (scaled != null) {
+            int dx = xpos + x - fOffsetX;
+            int dy = y - fOffsetY + 1;
+            g.drawImage(scaled, dx, dy);
+        }
+    }
 
     ustring title = a->getText();
 
@@ -623,15 +654,12 @@ void YListBox::paint(Graphics &g, const YRect &r) {
     for (int n(min); n <= max; n++) paintItem(g, n);
     resetScrollBars();
 
-    int const y(contentHeight());
+    unsigned const y(contentHeight());
 
     if (y < height()) {
-#ifdef CONFIG_GRADIENTS
         if (fGradient != null)
             g.drawImage(fGradient, 0, y, width(), height() - y, 0, y);
-        else 
-#endif
-            if (listbackPixmap != null)
+        else if (listbackPixmap != null)
             g.fillPixmap(listbackPixmap, 0, y, width(), height() - y);
         else {
             g.setColor(listBoxBg);
@@ -641,14 +669,17 @@ void YListBox::paint(Graphics &g, const YRect &r) {
 }
 
 void YListBox::paintItem(int i) {
-/// TODO #warning "fix this to use an invalidate region"
-    if (i >= 0 && i < getItemCount()) {
-        paintExpose(0, i * getLineHeight() - fOffsetY,
-                    width(), getLineHeight());
+    if (!fGraphics)
+        return;
+
+    if (i >= 0 && i < getItemCount() && fVisible) {
+        int y = i * getLineHeight() - fOffsetY;
+        YRect r(0, y, width(), getLineHeight());
+        fGraphics.paint(r);
     }
 }
 
-void YListBox::activateItem(YListItem */*item*/) {
+void YListBox::activateItem(YListItem *item) {
 }
 
 void YListBox::repaintItem(YListItem *item) {
@@ -842,15 +873,16 @@ bool YListBox::isSelected(YListItem *item) { // !!! remove this !!!
     return s;
 }
 
-int YListBox::contentWidth() {
+unsigned YListBox::contentWidth() {
     return maxWidth();
 }
 
-int YListBox::contentHeight() {
+unsigned YListBox::contentHeight() {
     return getItemCount() * getLineHeight();
 }
 
 YWindow *YListBox::getWindow() {
     return this;
 }
-#endif
+
+// vim: set sw=4 ts=4 et:
