@@ -42,25 +42,27 @@ YLocale::YLocale(char const * localeName) {
     instance = this;
 
     if (NULL == (fLocaleName = setlocale(LC_ALL, localeName))
-        /* || False == XSupportsLocale()*/) {
-        warn(_("Locale not supported by C library. "
+        || False == XSupportsLocale()) {
+        warn(_("Locale not supported by C library or Xlib. "
                "Falling back to 'C' locale'."));
         fLocaleName = setlocale(LC_ALL, "C");
     }
-#ifndef NO_CONFIGURE
     multiByte = true;
-#endif
 
     char const * codeset = NULL;
     int const codesetItems[] = {
 #ifdef CONFIG_NL_CODESETS
-	CONFIG_NL_CODESETS
+        CONFIG_NL_CODESETS
 #else
-	CODESET, _NL_CTYPE_CODESET_NAME, 0
+        CODESET,
+#ifdef _NL_CTYPE_CODESET_NAME
+        _NL_CTYPE_CODESET_NAME,
+#endif
+        0
 #endif
     };
 
-    for (unsigned int i = 0; 
+    for (unsigned int i = 0;
          i + 1 < ACOUNT(codesetItems)
          && NULL != (codeset = nl_langinfo(codesetItems[i]))
          && '\0' == *codeset;
@@ -74,16 +76,16 @@ YLocale::YLocale(char const * localeName) {
 
     union { int i; char c[sizeof(int)]; } endian; endian.i = 1;
 
-    MSG(("locale: %s, MB_CUR_MAX: %d, "
+    MSG(("locale: %s, MB_CUR_MAX: %zd, "
          "multibyte: %d, codeset: %s, endian: %c",
-         fLocaleName, MB_CUR_MAX, multiByte, codeset, endian.c ? 'b' : 'l'));
+         fLocaleName, MB_CUR_MAX, multiByte, codeset, *endian.c ? 'l' : 'b'));
 
 /// TODO #warning "this is getting way too complicated"
 
     char const * unicodeCharsets[] = {
 #ifdef CONFIG_UNICODE_SET
         CONFIG_UNICODE_SET,
-#endif    
+#endif
 //      "WCHAR_T//TRANSLIT",
         (*endian.c ? "UCS-4LE//TRANSLIT" : "UCS-4BE//TRANSLIT"),
 //      "WCHAR_T",
@@ -114,10 +116,8 @@ YLocale::YLocale(char const * localeName) {
     delete[] localeCharsets[0];
 #endif
 
-#ifdef ENABLE_NLS
     bindtextdomain(PACKAGE, LOCDIR);
     textdomain(PACKAGE);
-#endif
 }
 
 YLocale::~YLocale() {
@@ -126,7 +126,7 @@ YLocale::~YLocale() {
 
     if ((iconv_t) -1 != toUnicode) iconv_close(toUnicode);
     if ((iconv_t) -1 != toLocale) iconv_close(toLocale);
-#endif    
+#endif
 }
 
 #ifdef CONFIG_I18N
@@ -143,7 +143,7 @@ iconv_t YLocale::getConverter (const char *from, const char **&to) {
 /*
 YLChar * YLocale::localeString(YUChar const * uStr) {
     YLChar * lStr(NULL);
-    
+
     return lStr;
 }
 */
@@ -159,22 +159,26 @@ YUChar *YLocale::unicodeString(const YLChar *lStr, size_t const lLen,
     char * inbuf((char *) lStr), * outbuf((char *) uStr);
     size_t inlen(lLen), outlen(4 * lLen);
 
+#if defined(__NetBSD__) || defined(__sun)
+    if (0 > (int) iconv(instance->toUnicode, const_cast<const char **>(&inbuf), &inlen, &outbuf, &outlen))
+#else
     if (0 > (int) iconv(instance->toUnicode, &inbuf, &inlen, &outbuf, &outlen))
+#endif
         warn(_("Invalid multibyte string \"%s\": %s"), lStr, strerror(errno));
 
     *((YUChar *) outbuf) = 0;
     uLen = ((YUChar *) outbuf) - uStr;
-    
+
     return uStr;
 }
 #endif
 
-const char *YLocale::getLocaleName() { 
+const char *YLocale::getLocaleName() {
 #ifdef CONFIG_I18N
-    return instance->fLocaleName; 
+    return instance->fLocaleName;
 #else
     return "C";
-#endif    
+#endif
 }
 
 int YLocale::getRating(const char *localeStr) {
@@ -186,3 +190,5 @@ int YLocale::getRating(const char *localeStr) {
 
     return s2 - localeStr;
 }
+
+// vim: set sw=4 ts=4 et:
