@@ -7,8 +7,6 @@
  */
 #include "config.h"
 
-#ifndef LITE
-
 #include "ylib.h"
 #include "ymsgbox.h"
 
@@ -27,7 +25,6 @@ YMsgBox::YMsgBox(int buttons, YWindow *owner): YDialog(owner) {
     fButtonOK = 0;
     fButtonCancel = 0;
     fLabel = new YLabel(null, this);
-    fLabel->show();
 
     setToplevel(true);
 
@@ -49,13 +46,9 @@ YMsgBox::YMsgBox(int buttons, YWindow *owner): YDialog(owner) {
         }
     }
     autoSize();
-#if defined(GNOME1_HINTS) || defined(WMSPEC_HINTS)
     setWinLayerHint(WinLayerAboveDock);
-#endif
-#if defined(GNOME1_HINTS)
+    setWinWorkspaceHint(-1);
     setWinHintsHint(WinHintsSkipWindowMenu);
-#endif
-    setWinStateHint(WinStateAllWorkspaces, WinStateAllWorkspaces);
     {
 
         Atom protocols[2];
@@ -64,20 +57,12 @@ YMsgBox::YMsgBox(int buttons, YWindow *owner): YDialog(owner) {
         XSetWMProtocols(xapp->display(), handle(), protocols, 2);
         getProtocols(true);
     }
-    {
-        MwmHints mwm;
-
-        memset(&mwm, 0, sizeof(mwm));
-        mwm.flags =
-            MWM_HINTS_FUNCTIONS |
-            MWM_HINTS_DECORATIONS;
-        mwm.functions =
-            MWM_FUNC_MOVE | MWM_FUNC_CLOSE;
-        mwm.decorations =
-            MWM_DECOR_BORDER | MWM_DECOR_TITLE | MWM_DECOR_MENU;
-
-        setMwmHints(mwm);
-    }
+    setMwmHints(MwmHints(
+       MWM_HINTS_FUNCTIONS | MWM_HINTS_DECORATIONS,
+       MWM_FUNC_MOVE | MWM_FUNC_CLOSE,
+       MWM_DECOR_BORDER | MWM_DECOR_TITLE | MWM_DECOR_MENU,
+       0,
+       0));
 }
 
 YMsgBox::~YMsgBox() {
@@ -87,22 +72,22 @@ YMsgBox::~YMsgBox() {
 }
 
 void YMsgBox::autoSize() {
-    int lw = fLabel ? fLabel->width() : 0;
-    int w = lw + 24, h;
+    unsigned lw = fLabel ? fLabel->width() : 0;
+    unsigned w = lw + 24, h;
+
+    w = clamp(w, 240U, desktop->width());
 
-    w = clamp(w, 240, desktop->width());
-    
     h = 12;
     if (fLabel) {
         fLabel->setPosition((w - lw) / 2, h);
         h += fLabel->height();
     }
     h += 18;
-    
+
     unsigned const hh(max(fButtonOK ? fButtonOK->height() : 0,
                           fButtonCancel ? fButtonCancel->height() : 0));
     unsigned const ww(max(fButtonOK ? fButtonOK->width() : 0,
-                          fButtonCancel ? fButtonCancel->width() : 0) + 3);
+                          fButtonCancel ? fButtonCancel->width() : 3));
 
     if (fButtonOK) {
         fButtonOK->setSize(ww, hh);
@@ -116,7 +101,7 @@ void YMsgBox::autoSize() {
     h += fButtonOK ? fButtonOK->height() :
         fButtonCancel ? fButtonCancel->height() : 0;
     h += 12;
-    
+
     setSize(w, h);
 }
 
@@ -127,32 +112,47 @@ void YMsgBox::setTitle(const ustring &title) {
 }
 
 void YMsgBox::setText(const ustring &text) {
-    if (fLabel)
+    if (fLabel) {
         fLabel->setText(text);
+        fLabel->show();
+    }
     autoSize();
 }
 
 void YMsgBox::setPixmap(ref<YPixmap>/*pixmap*/) {
 }
 
-void YMsgBox::actionPerformed(YAction *action, unsigned int /*modifiers*/) {
+void YMsgBox::actionPerformed(YAction action, unsigned int /*modifiers*/) {
     if (fListener) {
-        if (action == fButtonOK) {
+        if (fButtonOK && action == *fButtonOK) {
             fListener->handleMsgBox(this, mbOK);
-        } else if (action == fButtonCancel) {
+        }
+        else if (fButtonCancel && action == *fButtonCancel) {
             fListener->handleMsgBox(this, mbCancel);
         }
+        else {
+            TLOG(("unknown action %d for msgbox", action.ident()));
+        }
     }
 }
 
 void YMsgBox::handleClose() {
-    fListener->handleMsgBox(this, 0);
+    if (fListener)
+        fListener->handleMsgBox(this, 0);
+    else {
+        manager->unmanageClient(this);
+        manager->focusTopWindow();
+    }
 }
 
 void YMsgBox::handleFocus(const XFocusChangeEvent &/*focus*/) {
 }
 
 void YMsgBox::showFocused() {
+    YRect r(desktop->getScreenGeometry());
+    setPosition(r.x() + int(r.width() / 2) - int(width() / 2),
+                r.y() + int(r.height() / 2) - int(height() / 2));
+
     switch (msgBoxDefaultAction) {
     case 0:
         if (fButtonCancel) fButtonCancel->requestFocus(false);
@@ -161,15 +161,12 @@ void YMsgBox::showFocused() {
         if (fButtonOK) fButtonOK->requestFocus(false);
         break;
     }
-    if (getFrame() == 0)
+    if (getFrame() == nullptr) {
         manager->manageClient(handle(), false);
+    }
     if (getFrame()) {
-        int dx, dy, dw, dh;
-        desktop->getScreenGeometry(&dx, &dy, &dw, &dh);
-        getFrame()->setNormalPositionOuter(
-            dx + dw / 2 - getFrame()->width() / 2,
-            dy + dh / 2 - getFrame()->height() / 2);
         getFrame()->activateWindow(true);
     }
 }
-#endif
+
+// vim: set sw=4 ts=4 et:
