@@ -16,8 +16,6 @@
 #endif
 
 static inline Display* display()  { return xapp->display(); }
-static inline Colormap colormap() { return xapp->colormap(); }
-static inline Visual*  visual()   { return xapp->visual(); }
 
 /******************************************************************************/
 
@@ -116,12 +114,10 @@ Graphics::~Graphics() {
 
 #ifdef CONFIG_XFREETYPE
 XftDraw* Graphics::handleXft() {
-    if (fXftDraw == nullptr && rdepth() == 32 && xapp->alpha()) {
-        fXftDraw = XftDrawCreateAlpha(display(), drawable(), 32);
-    }
     if (fXftDraw == nullptr) {
         fXftDraw = XftDrawCreate(display(), drawable(),
-                    visual(), colormap());
+                                 xapp->visualForDepth(rdepth()),
+                                 xapp->colormapForDepth(rdepth()));
     }
     return fXftDraw;
 }
@@ -133,6 +129,8 @@ Picture Graphics::picture() {
         if (format) {
             XRenderPictureAttributes attr;
             unsigned long mask = None;
+            attr.component_alpha = (rDepth == 32);
+            mask |= CPComponentAlpha;
             fPicture = XRenderCreatePicture(display(), fDrawable,
                                             format, mask, &attr);
         }
@@ -149,14 +147,9 @@ void Graphics::clear()
 
 void Graphics::clearArea(int x, int y, unsigned w, unsigned h)
 {
-    XGCValues val;                      unsigned long mask = 0;
-    val.foreground         = None;      mask |= GCForeground;
-    val.background         = None;      mask |= GCBackground;
-    val.fill_style         = FillSolid; mask |= GCFillStyle;
-    val.function           = GXcopy;    mask |= GCFunction;
-    val.graphics_exposures = False;     mask |= GCGraphicsExposures;
-    XChangeGC(display(), gc, mask, &val);
+    setFunction(GXclear);
     XFillRectangle(display(), drawable(), gc, x, y, w, h);
+    setFunction(GXcopy);
 }
 
 void Graphics::copyArea(const int x, const int y,
@@ -515,16 +508,22 @@ void Graphics::setLineWidth(unsigned width) {
 
 void Graphics::setPenStyle(bool dotLine) {
     XGCValues gcv;
+    unsigned long mask = GCLineStyle;
+    gcv.line_style = dotLine ? LineOnOffDash : LineSolid;
 
     if (dotLine) {
-        char c = 1;
-        gcv.line_style = LineOnOffDash;
-        XSetDashes(display(), gc, 0, &c, 1);
-    } else {
-        gcv.line_style = LineSolid;
+        char dashes[] = { 1 };
+        int num_dashes = int ACOUNT(dashes);
+        int dash_offset = 0;
+        XSetDashes(display(), gc, dash_offset, dashes, num_dashes);
+
+        gcv.line_width = 1;
+        gcv.cap_style = CapButt;
+        gcv.join_style = JoinMiter;
+        mask |= GCLineWidth | GCCapStyle | GCJoinStyle;
     }
 
-    XChangeGC(display(), gc, GCLineStyle, &gcv);
+    XChangeGC(display(), gc, mask, &gcv);
 }
 
 void Graphics::setFunction(int function) {
@@ -540,7 +539,7 @@ void Graphics::drawImage(ref<YImage> img, int const x, int const y) {
 void Graphics::drawImage(ref<YImage> img, int x, int y, unsigned w, unsigned h, int dx, int dy) {
     if (picture()) {
         unsigned depth = max(img->depth(), rdepth());
-        ref<YPixmap> pix(img->renderToPixmap(depth));
+        ref<YPixmap> pix(img->renderToPixmap(depth, img->depth() == 32));
         if (pix != null) {
             Picture source = pix->picture();
             XRenderComposite(display(),
@@ -638,7 +637,7 @@ void Graphics::compositeImage(ref<YImage> img, int const sx, int const sy, unsig
             return;
 
         unsigned depth = max(img->depth(), rdepth());
-        ref<YPixmap> pix(img->renderToPixmap(depth));
+        ref<YPixmap> pix(img->renderToPixmap(depth, img->depth() == 32));
         if (pix != null) {
             Picture source = pix->picture();
             XRenderComposite(display(),
@@ -1097,7 +1096,7 @@ void GraphicsBuffer::paint(Pixmap pixmap, const YRect& rect) {
 
     Graphics gfx(pixmap, w, h, depth);
 
-    if (clipping) {
+    if (clipping && fNesting == 1) {
         if (x || y || w < window()->width() || h < window()->height()) {
             XRectangle clip = { short(x), short(y),
                                (unsigned short)w, (unsigned short)h };
@@ -1105,6 +1104,10 @@ void GraphicsBuffer::paint(Pixmap pixmap, const YRect& rect) {
         }
     }
 
+    if (fNesting == 1) {
+        gfx.clearArea(x, y, w, h);
+    }
+
     window()->paint(gfx, rect);
 
     if (pixmap == fPixmap && fNesting == 1) {
