@@ -22,15 +22,15 @@ static char *sessionProg;
 upath getsesfile() {
     upath path(YApplication::getPrivConfDir());
     if (false == path.dirExists())
-        path.mkdir(0755);
-    path += mstring("/.session-") + newSessionId;
+        path.mkdir();
+    path += mstring("/.session-", newSessionId);
     return path;
 }
 
 static void iceWatchFD(IceConn conn,
                        IcePointer /*client_data*/,
                        Bool opening,
-                       IcePointer */*watch_data*/)
+                       IcePointer* /*watch_data*/)
 {
     if (opening) {
         if (IceSMfd != -1) { // shouldn't happen
@@ -265,3 +265,5 @@ bool YSMPoll::forWrite() {
 }
 
 #endif /* CONFIG_SESSION */
+
+// vim: set sw=4 ts=4 et:
