@@ -12,12 +12,7 @@ class YToolTip;
 class YTimer;
 class YAutoScroll;
 class YRect;
-
-#ifdef XINERAMA
-extern "C" {
-#include <X11/extensions/Xinerama.h>
-}
-#endif
+class YRect2;
 
 struct DesktopScreenInfo {
     int screen_number;
@@ -25,14 +20,32 @@ struct DesktopScreenInfo {
     int y_org;
     unsigned width;
     unsigned height;
+
+    DesktopScreenInfo(int i, int x, int y, unsigned w, unsigned h) :
+        screen_number(i), x_org(x), y_org(y), width(w), height(h)
+    { }
+    operator YRect() const {
+        return YRect(x_org, y_org, width, height);
+    }
+    unsigned horizontal() const {
+        return width + unsigned(x_org);
+    }
+    unsigned vertical() const {
+        return height + unsigned(y_org);
+    }
 };
 
 class YWindow : protected YWindowList, private YWindowNode {
 public:
-    YWindow(YWindow *aParent = 0, Window win = 0, int depth = CopyFromParent, Visual *visual = CopyFromParent);
+    YWindow(YWindow *aParent = nullptr,
+            Window window = None,
+            int depth = CopyFromParent,
+            Visual *visual = nullptr,
+            Colormap colormap = CopyFromParent);
     virtual ~YWindow();
 
     void setStyle(unsigned aStyle);
+    void addStyle(unsigned aStyle) { setStyle(fStyle | aStyle); }
     unsigned getStyle() const { return fStyle; }
     long getEventMask() const { return fEventMask; }
     void addEventMask(long mask);
@@ -43,9 +56,10 @@ public:
     virtual void raise();
     virtual void lower();
 
-    void repaint();
-    void repaintFocus();
-    void repaintSync();
+    virtual void repaint();
+    virtual void repaintFocus();
+    virtual void repaintSync();
+
     void readAttributes();
     void reparent(YWindow *parent, int x, int y);
     bool getWindowAttributes(XWindowAttributes* attr);
@@ -61,13 +75,13 @@ public:
     void setPosition(int x, int y);
     void setBorderWidth(unsigned width);
     void setBackground(unsigned long pixel);
+    void setBackgroundPixmap(ref<YPixmap> pixmap);
     void setBackgroundPixmap(Pixmap pixmap);
-    void setParentRelative(void);
+    void setParentRelative();
     virtual void configure(const YRect &r);
-
+    virtual void configure(const YRect2& r2);
 
     virtual void paint(Graphics &g, const YRect &r);
-    virtual void paintFocus(Graphics &, const YRect &) {}
 
     virtual void handleEvent(const XEvent &event);
 
@@ -97,11 +111,13 @@ public:
     virtual void handleReparentNotify(const XReparentEvent &) {}
     virtual void handleConfigureRequest(const XConfigureRequestEvent &);
     virtual void handleMapRequest(const XMapRequestEvent &);
+    virtual void handleDamageNotify(const XDamageNotifyEvent &) {}
 #ifdef CONFIG_SHAPE
     virtual void handleShapeNotify(const XShapeEvent &) {}
 #endif
 #ifdef CONFIG_XRANDR
-    virtual void handleRRScreenChangeNotify(const XRRScreenChangeNotifyEvent &/*xrrsc*/) {}
+    virtual void handleRRScreenChangeNotify(const XRRScreenChangeNotifyEvent &) {}
+    virtual void handleRRNotify(const XRRNotifyEvent &) {}
 #endif
 
     virtual void handleClickDown(const XButtonEvent &, int) {}
@@ -110,8 +126,6 @@ public:
     virtual void handleDrag(const XButtonEvent &, const XMotionEvent &) {}
     virtual void handleEndDrag(const XButtonEvent &, const XButtonEvent &) {}
 
-    virtual void handleEndPopup(YPopupWindow *popup);
-
     virtual void handleClose();
 
     virtual bool handleAutoScroll(const XMotionEvent &mouse);
@@ -149,13 +163,17 @@ public:
     unsigned height() const { return fHeight; }
     unsigned depth() const { return fDepth; }
     Visual *visual() const { return fVisual; }
+    Colormap colormap();
     YRect geometry() const { return YRect(fX, fY, fWidth, fHeight); }
+    YDimension dimension() const { return YDimension(fWidth, fHeight); }
 
     bool visible() const { return (flags & wfVisible); }
     bool created() const { return (flags & wfCreated); }
     bool adopted() const { return (flags & wfAdopted); }
-    bool destroyed() const { return (flags & wfDestroyed); }
     bool focused() const { return (flags & wfFocused); }
+    bool destroyed() const { return (flags & wfDestroyed); }
+    void setDestroyed();
+    bool testDestroyed();
 
     virtual void donePopup(YPopupWindow * /*command*/);
 
@@ -164,7 +182,7 @@ public:
         wsSaveUnder        = 1 << 1,
         wsManager          = 1 << 2,
         wsInputOnly        = 1 << 3,
-        wsOutputOnly       = 1 << 4,
+        wsNoExpose         = 1 << 4,
         wsPointerMotion    = 1 << 5,
         wsDesktopAware     = 1 << 6,
         wsToolTip          = 1 << 7,
@@ -177,7 +195,7 @@ public:
     void nextFocus();
     void prevFocus();
     bool changeFocus(bool next);
-    void requestFocus(bool requestUserFocus);
+    virtual void requestFocus(bool requestUserFocus);
     void setFocus(YWindow *window);
     YWindow *getFocusWindow();
     virtual void gotFocus();
@@ -199,6 +217,11 @@ public:
     void setWinGravity(int gravity);
     void setBitGravity(int gravity);
 
+    void setProperty(Atom prop, Atom type, const Atom* values, int count);
+    void setProperty(Atom property, Atom propType, Atom value);
+    void setNetWindowType(Atom window_type);
+    void setNetOpacity(Atom opacity);
+    void setNetPid();
     void setDND(bool enabled);
 
     void XdndStatus(bool acceptDrop, Atom dropAction);
@@ -212,6 +235,11 @@ public:
     int getClickCount() { return fClickCount; }
 
     void scrollWindow(int dx, int dy);
+    void clearWindow();
+    void clearArea(int x, int y, unsigned w, unsigned h, bool expos = false);
+    Pixmap createPixmap();
+    XRenderPictFormat* format();
+    Picture createPicture();
 
     bool toolTipVisible();
     virtual void updateToolTip();
@@ -248,7 +276,7 @@ private:
 
     unsigned fDepth;
     Visual *fVisual;
-    Colormap fAllocColormap;
+    Colormap fColormap;
 
     YWindow *fParentWindow;
     YWindow *fFocusedWindow;
@@ -305,8 +333,10 @@ public:
     YDesktop(YWindow *aParent = 0, Window win = 0);
     virtual ~YDesktop();
 
-    void updateXineramaInfo(unsigned &w, unsigned &h);
+    bool updateXineramaInfo(unsigned& horizontal, unsigned& vertical);
 
+    const DesktopScreenInfo& getScreenInfo(int screen_no = -1);
+    YRect getScreenGeometry(int screen_no = -1);
     void getScreenGeometry(int *x, int *y,
                            unsigned *width, unsigned *height,
                            int screen_no = -1);
@@ -322,20 +352,29 @@ protected:
 
 extern YDesktop *desktop;
 
-#ifdef CONFIG_SHAPE
-extern int shapesSupported;
-extern int shapeEventBase, shapeErrorBase;
-#endif
+struct YExtension {
+    int eventBase, errorBase;
+    int versionMajor, versionMinor;
+    bool supported;
 
-#ifdef CONFIG_XRANDR
-extern int xrandrSupported;
-extern bool xrandr12;
-extern int xrandrEventBase, xrandrErrorBase;
-#endif
+    typedef int (*QueryFunc)(Display *, int *, int *);
+    void init(Display* dis, QueryFunc ext, QueryFunc ver);
+    bool isEvent(int type, int eventNumber) const {
+        return supported && type == eventBase + eventNumber;
+    }
+};
 
+extern YExtension composite;
+extern YExtension damage;
+extern YExtension fixes;
+extern YExtension render;
+extern YExtension shapes;
+extern YExtension xrandr;
+extern YExtension xinerama;
 
 extern Atom _XA_WM_CHANGE_STATE;
 extern Atom _XA_WM_CLASS;
+extern Atom _XA_WM_CLIENT_LEADER;
 extern Atom _XA_WM_COLORMAP_NOTIFY;
 extern Atom _XA_WM_COLORMAP_WINDOWS;
 extern Atom _XA_WM_COMMAND;
@@ -356,7 +395,9 @@ extern Atom _XA_WM_ZOOM_HINTS;
 
 extern Atom _XATOM_MWM_HINTS;
 extern Atom _XA_CLIPBOARD;
+extern Atom _XA_MANAGER;
 extern Atom _XA_TARGETS;
+extern Atom _XA_XEMBED;
 extern Atom _XA_XEMBED_INFO;
 
 /* Xdnd */
@@ -369,6 +410,30 @@ extern Atom XA_XdndStatus;
 extern Atom XA_XdndDrop;
 extern Atom XA_XdndFinished;
 
+extern Atom _XA_KDE_NET_SYSTEM_TRAY_WINDOWS;
+extern Atom _XA_NET_SYSTEM_TRAY_OPCODE;
+extern Atom _XA_NET_SYSTEM_TRAY_ORIENTATION;
+extern Atom _XA_NET_SYSTEM_TRAY_MESSAGE_DATA;
+extern Atom _XA_NET_SYSTEM_TRAY_VISUAL;
+extern Atom _XA_NET_WM_NAME;
+extern Atom _XA_NET_WM_PID;
+extern Atom _XA_NET_WM_WINDOW_OPACITY;              // OK
+extern Atom _XA_NET_WM_WINDOW_TYPE;                 // OK
+extern Atom _XA_NET_WM_WINDOW_TYPE_COMBO;           // OK
+extern Atom _XA_NET_WM_WINDOW_TYPE_DESKTOP;         // OK
+extern Atom _XA_NET_WM_WINDOW_TYPE_DIALOG;          // OK
+extern Atom _XA_NET_WM_WINDOW_TYPE_DND;             // OK
+extern Atom _XA_NET_WM_WINDOW_TYPE_DOCK;            // OK
+extern Atom _XA_NET_WM_WINDOW_TYPE_DROPDOWN_MENU;   // OK
+extern Atom _XA_NET_WM_WINDOW_TYPE_MENU;            // OK
+extern Atom _XA_NET_WM_WINDOW_TYPE_NORMAL;          // OK
+extern Atom _XA_NET_WM_WINDOW_TYPE_NOTIFICATION;    // OK
+extern Atom _XA_NET_WM_WINDOW_TYPE_POPUP_MENU;      // OK
+extern Atom _XA_NET_WM_WINDOW_TYPE_SPLASH;          // OK
+extern Atom _XA_NET_WM_WINDOW_TYPE_TOOLBAR;         // OK
+extern Atom _XA_NET_WM_WINDOW_TYPE_TOOLTIP;         // OK
+extern Atom _XA_NET_WM_WINDOW_TYPE_UTILITY;         // OK
+
 #endif
 
 // vim: set sw=4 ts=4 et:
