@@ -81,6 +81,7 @@ private:
     YAtom _NET_SYSTEM_TRAY_S0;
     YXTray *fTray;
     lazy<YTimer> fUpdateTimer;
+    YObjectArray<DockRequest> fDockRequests;
     typedef YObjectArray<DockRequest>::IterType DockIter;
     mstring toolTip;
 
@@ -111,7 +112,7 @@ YXTrayProxy::YXTrayProxy(const YAtom& atom, YXTray *tray):
     _NET_SYSTEM_TRAY_S0(atom),
     fTray(tray)
 {
-    setStyle(wsNoExpose);
+    addStyle(wsNoExpose);
     setTitle("YXTrayProxy");
     if (isExternal()) {
         long orientation = SYSTEM_TRAY_ORIENTATION_HORZ;
@@ -193,7 +194,7 @@ void YXTrayProxy::expireMessages() {
 void YXTrayProxy::updateToolTip() {
     MSG(("YXTrayProxy::updateToolTip"));
     long size = 0;
-    if (messages.getCount() > 0) {
+    if (messages.nonempty()) {
         expireMessages();
         for (IterType iter = messages.iterator(); ++iter; ) {
             if (iter->offset > 0) {
@@ -250,6 +251,13 @@ bool YXTrayProxy::handleTimer(YTimer *timer) {
         return false;
     }
 
+    DockIter dock = fDockRequests.iterator();
+    while (++dock && timer != dock->timer);
+    if (dock) {
+        fTray->trayRequestDock(dock->window, dock->title);
+        dock.remove();
+    }
+
     return false;
 }
 
@@ -282,6 +290,13 @@ bool YXTrayProxy::requestDock(Window win) {
         return false;
     }
 
+    if (title == "Pidgin") {
+        long delay = 200L + 25L * fDockRequests.getCount();
+        YTimer* tm = new YTimer(delay, this, true, true);
+        fDockRequests.append(new DockRequest(win, tm, title));
+        return true;
+    }
+
     return fTray->trayRequestDock(win, title);
 }
 
@@ -348,7 +363,8 @@ YXTrayEmbedder::YXTrayEmbedder(YXTray *tray, Window win, Window ldr, cstring tit
     fLeader(Elvis(ldr, win)),
     fTitle(title),
     fDamage(None),
-    fComposing(xapp->format() && damage.supported && composite.supported),
+    fComposing(xapp->alpha() && composite.supported &&
+               xapp->format() && damage.supported),
     fOrder(getOrder(fTitle))
 {
     if (fClient->destroyed())
@@ -367,31 +383,9 @@ YXTrayEmbedder::YXTrayEmbedder(YXTray *tray, Window win, Window ldr, cstring tit
         XCompositeRedirectWindow(xapp->display(), win, CompositeRedirectManual);
     }
 
-    Atom info[] = { XEMBED_PROTOCOL_VERSION, XEMBED_MAPPED };
-    fClient->setProperty(_XA_XEMBED_INFO, XA_CARDINAL, info, 2);
-
-    XClientMessageEvent xev = {};
-    xev.type = ClientMessage;
-    xev.window = win;
-    xev.message_type = _XA_XEMBED;
-    xev.format = 32;
-    xev.data.l[0] = CurrentTime;
-    xev.data.l[1] = XEMBED_EMBEDDED_NOTIFY;
-    xev.data.l[2] = 0; // no detail
-    xev.data.l[3] = handle();
-    xev.data.l[4] = XEMBED_PROTOCOL_VERSION;
-    xapp->send(xev, win, NoEventMask);
-
-    xev.type = ClientMessage;
-    xev.window = win;
-    xev.message_type = _XA_XEMBED;
-    xev.format = 32;
-    xev.data.l[0] = CurrentTime;
-    xev.data.l[1] = XEMBED_WINDOW_ACTIVATE;
-    xev.data.l[2] = 0; // no detail
-    xev.data.l[3] = 0; // no data1
-    xev.data.l[4] = 0; // no data2
-    xapp->send(xev, win, NoEventMask);
+    fClient->infoMapped();
+    fClient->sendNotify();
+    fClient->sendActivate();
 
     if (xapp->alpha() == false)
         fClient->setParentRelative();
@@ -502,7 +496,7 @@ YXTray::YXTray(YXTrayNotifier *notifier,
     fRunProxy(internal == false),
     fDrawBevel(drawBevel)
 {
-    setStyle(wsNoExpose);
+    addStyle(wsNoExpose);
     setTitle("YXTray");
     setParentRelative();
     fTrayProxy = new YXTrayProxy(atom, this);
@@ -775,7 +769,7 @@ void YXTray::trayUpdateGeometry(unsigned w, unsigned h, bool visible) {
 }
 
 bool YXTray::kdeRequestDock(Window win) {
-    if (fDocked.getCount() == 0)
+    if (fDocked.isEmpty())
         return false;
     puts("trying to dock");
     YAtom _NET_SYSTEM_TRAY_S0("_NET_SYSTEM_TRAY_S", true);
